package RRD::Info;

#
# rrdbrowse info file reading/writing functions
#
# $Id: Info.pm,v 1.26 2003/06/19 08:18:54 tommy Exp $
#

use strict;
use Carp;
use Sys::Syslog qw(:DEFAULT setlogsock);
use RRDs;
use RRD::Config;
use SNMP_util;
use POSIX qw(strftime);
use SNMP_Session;
use Exporter;

$SNMP_Session::suppress_warnings = 2;

use vars     qw($VERSION @allowed @ISA @EXPORT_OK
                $intfile $intcacheto $AUTOLOAD %autosubs); 
@ISA 	   = qw(Exporter);
@EXPORT_OK = qw(LogEntry);

@allowed   = qw(Interface Target Bandwidth
		 Limit Description In Out Type
		 Connection Router CID Public ValueIP
		 Title OID ListNr Options HRule Send
		 Queue Status Users Disk
		);

my @DEFAULT_REQUIRED = qw(Type Target Description);

my @autofun = qw(RRD NFO OutDir Width Height TNWidth TNHeight);
for my $attr ( @allowed, @autofun ) { $autosubs{$attr}++; }

sub AUTOLOAD {
    my $self = shift;
    my $func = $AUTOLOAD;
    $func =~ s/^.*:://;

    return if $func =~ /^DESTROY$/;
    if ( $autosubs{$func} ) {
        $self->{$func} = shift if @_;
        return $self->{$func};
    }else{
        my $f = "SUPER::$func";
        $self->$f(@_);
    }
}

sub new {
    my ($pkg, $file) = @_;

    #
    # setup reasonable default values
    #
    my $self = { "Type"     => "port",
		 "In"	    => "Incoming Traffic",
		 "Out"	    => "Outgoing Traffic",
		 "Width"    => 600,
	  	 "Height"   => 260,
		 "TNWidth"  => 220,
	  	 "TNHeight" => 100,
	       };

    #
    # load the info file creating a hash of its values
    #
    open(F, $file) || croak "$! ($file)";
    while(<F>) {
        chop;

	next if m/^#|^;|^$/;
        my ($key,$value) = split(/:\s+/,$_,2);
        if ( ! grep (/$key/,@allowed) ) {
            croak "Undefined variable $key in $file";
        }
        $self->{$key} = $value;
    }
    close F;

    #
    # check for more default values
    #
    if ( $file !~ s/$config{nfopath}//g ) {
        croak "NFO File not in nfopath\n";
    }
    $file =~ s/^\///g;
    $file =~ m/(.*)\/(.+).nf.*$/g;
    $self->{Workdir} = $1;
    $self->{Directory} = $2;
    if ( ! $1 || ! $2 ) { croak "(DEBUG) Invalid Path ($file, $1, $2)"; }


    $self->{RRD}    = "$config{rrdpath}/".$self->{Workdir}."/".
		       $self->{Directory}."/".$self->{Directory}.".rrd";

    $self->{OutDir} = "$config{pngpath}/".$self->{Workdir}."/".
		       $self->{Directory};

    $self->{NFO}    = "$config{nfopath}/".$self->{Workdir}."/".
		       $self->{Directory}.".nfo";

    #
    # create a client object of type Type
    #
    if ( ! eval 'require "RRD/Function/$self->{Type}.pm"' ) {
        die "Module $self->{Type} not found in $self->{NFO}\n" 
    }
    require "RRD/Function/$self->{Type}.pm";
    my $func = "RRD::Function::$self->{Type}";
    no strict 'refs';
    $self->{Function} = new {$func}($self);

    # check for required keys. 
    my $r = $func."::REQUIRED";
    for( @{$r}, @DEFAULT_REQUIRED ) {
        die "Required variable $_ missing in $self->{NFO}\n" if ! $self->{$_};
    }

    return bless { %{$self} }, $pkg;
}

sub Option {
    my ($self,$opt) = @_;

    return undef if ! defined $self->{Options};
    return grep(/$opt/i, split(/\s+/,$self->{Options}) ) ? 1 : undef;
}

sub CheckErrors {
    my ($self) = @_;

    my $error = RRDs::error;
    warn("Error in $self->{NFO} because $error\n") if $error;
}

sub CheckPaths {
    my ($self) = @_;
    umask(000);
 
    my $grp = $config{group} || "www-data";

    if ( !  -d "$config{pngpath}/".$self->{Workdir}."/".$self->{Directory} ) {
        `mkdir -p -m 02775 $config{pngpath}/$self->{Workdir}/$self->{Directory}`;
        `chgrp $grp $config{pngpath}/$self->{Workdir}/$self->{Directory}`;
    }

    if ( !  -d "$config{rrdpath}/".$self->{Workdir}."/".$self->{Directory} ) {
        `mkdir -p -m 02775 $config{rrdpath}/$self->{Workdir}/$self->{Directory}`;
    }

    if ( ! -f $self->{RRD} ) {
        $self->{Function}->CreateDB;
	$self->CheckErrors;
    }
}

sub LogEntry {
    my ($msg) = @_;
  
    if ( $config{logfacility} ) {
    	setlogsock('unix');
        openlog('rbupdate','pid',$config{logfacility});
        syslog('warning',$msg);
        closelog();
    }else{
    	warn($msg);
    }
}

sub Update {
    my ($self) = @_;

    if ( ! $self->{Function}->Update() ) {
        LogEntry("Error during update of $self->{NFO}\n") if ! $self->Option("nowarn");
    }

    $self->CheckErrors;
}

sub Graph {
    my ($self) = @_;

    return if ! -f $self->{RRD};

    $self->{Function}->Graph("1day",  -86400);
    $self->CheckErrors;

    $self->{Function}->Graph("1week", -604800);
    $self->CheckErrors;

    $self->{Function}->Graph("8weeks",-4838400);
    $self->CheckErrors;

    $self->{Function}->Graph("2years",-63072000);
    $self->CheckErrors;
}

sub GraphThumb {
    my ($self) = @_;

    return if ! -f $self->{RRD};

    $self->{Function}->Graph("1day", -86400);
    $self->CheckErrors;
}

1;

